#include "pico8.h"
#include "Tiles.h"
#include "Map.h"

uint8_t _color=WHITE;
int16_t _cameraX=0;
int16_t _cameraY=0;


#ifdef POKITTO
uint8_t _palette[16*3] = {0,0,0, 29,43,83, 126,37,83, 0,135,81, 171,82,54, 95,87,79, 194,195,199, 255,241,232, 255,0,77, 255,163,0, 255,240,36, 0,231,86, 41,173,255, 131,118,156, 255,119,168, 255,204,170 };
#elif defined PICOBOY
uint16_t _palette[16]={0x0000,0x194A,0x792A,0x042A,0xAA86,0x5AA9,0xC618,0xFF9D,0xF809,0xFD00,0xFF84,0x072A,0x2D7F,0x83B3,0xFBB5,0xFE75};
#endif // POKITTO


/*--------------------------------------------------------------------------------------------
	Graphics
--------------------------------------------------------------------------------------------*/


uint8_t pget (uint8_t x,uint8_t y)
{
    return pb.display.getPixel(X_OFFSET+x-_cameraX,Y_OFFSET+y-_cameraY);
};
void pset (uint8_t x,uint8_t y,uint8_t col)
{
    #ifdef POKITTO
    pb.display.setColor(col==0xFF?_color:col);
    pb.display.drawPixel(X_OFFSET+x-_cameraX,Y_OFFSET+y-_cameraY);
    #elif defined PICOBOY
    pb.display.drawPixel(X_OFFSET+x-_cameraX,Y_OFFSET+y-_cameraY,col==0xFF?_color:col);
    #endif // POKITTO
};
uint8_t sget (uint8_t x,uint8_t y)
{
    //TODO
    return 0;
};


uint8_t fget (uint8_t n,uint8_t f)
{
    if(f<8)
        return _gff[n] & 1<<f;

    return _gff[n];
};


void print (char *str)
{
    #ifdef POKITTO
    pb.display.setColor(_color);
    #elif defined PICOBOY
    pb.display.setTextColor(_color);
    #endif // POKITTO
    pb.display.print(str);
};
void print (char *str ,uint8_t x ,uint8_t y ,uint8_t col)
{
    pb.display.setCursor(X_OFFSET+x-_cameraX,Y_OFFSET+y-_cameraY);
    #ifdef POKITTO
    pb.display.setColor(col==0xFF?_color:col);
    #elif defined PICOBOY
    pb.display.setTextColor(col==0xFF?_color:col);
    #endif // POKITTO
    pb.display.print(str);
};



void color (uint8_t col)
{
    _color=col;
};
void cls (int8_t col)
{
    pb.display.fillScreen(col);
};
void camera (int16_t x, int16_t y)
{
    _cameraX=x;
    _cameraY=y;
};

void circ (uint8_t x,uint8_t y,uint8_t r,uint8_t col)
{
    #ifdef POKITTO
    pb.display.setColor(col==0xFF?_color:col);
    pb.display.drawCircle(X_OFFSET+x-_cameraX,Y_OFFSET+y-_cameraY,r);
    #elif defined PICOBOY
    pb.display.drawCircle(x-_cameraX,y-_cameraY,r,col==0xFF?_color:col);
    #endif // POKITTO
};
void circfill (uint8_t x,uint8_t y,uint8_t r,uint8_t col)
{
    #ifdef POKITTO
    pb.display.setColor(col==0xFF?_color:col);
    pb.display.fillCircle(X_OFFSET+x-_cameraX,Y_OFFSET+y-_cameraY,r);
    #elif defined PICOBOY
    pb.display.fillCircle(x-_cameraX,y-_cameraY,r,col==0xFF?_color:col);
    #endif // POKITTO
};

void line (uint8_t x0, uint8_t y0, uint8_t x1 ,uint8_t y1, uint8_t col)
{
    #ifdef POKITTO
    pb.display.setColor(col==0xFF?_color:col);
    pb.display.drawLine(X_OFFSET+x0-_cameraX,Y_OFFSET+y0-_cameraY,X_OFFSET+x1-_cameraX,Y_OFFSET+y1-_cameraY);
    #elif defined PICOBOY
    pb.display.drawLine(x0-_cameraX,y0-_cameraY,x1-_cameraX,y1-_cameraY,col==0xFF?_color:col);
    #endif // POKITTO
};
void rect (uint8_t x0, uint8_t y0, uint8_t x1 ,uint8_t y1, uint8_t col)
{
    #ifdef POKITTO
    pb.display.setColor(col==0xFF?_color:col);
    pb.display.drawRect(X_OFFSET+x0-_cameraX,Y_OFFSET+y0-_cameraY,abs((x1-_cameraX)-(x0-_cameraX)),abs((y1-_cameraY)-(y0-_cameraY)));
    #elif defined PICOBOY
    pb.display.drawRect(x0-_cameraX,y0-_cameraY,abs((x1-_cameraX)-(x0-_cameraX)),abs((y1-_cameraY)-(y0-_cameraY)),col==0xFF?_color:col);
    #endif // POKITTO
};
void rectfill (uint8_t x0, uint8_t y0, uint8_t x1 ,uint8_t y1, uint8_t col)
{
    #ifdef POKITTO
    pb.display.setColor(col==0xFF?_color:col);
    pb.display.fillRect(X_OFFSET+x0-_cameraX,Y_OFFSET+y0-_cameraY,abs((x1-_cameraX)-(x0-_cameraX)),abs((y1-_cameraY)-(y0-_cameraY)));
    #elif defined PICOBOY
    pb.display.fillRect(x0-_cameraX,y0-_cameraY,abs((x1-_cameraX)-(x0-_cameraX)),abs((y1-_cameraY)-(y0-_cameraY)),col==0xFF?_color:col);
    #endif // POKITTO
};
void pal(uint8_t c0,uint8_t c1,uint8_t p)
{
    if(p==0)
    {
        #ifdef POKITTO
        _palette[c0*3]=palettePico[c1*3];
        _palette[c0*3+1]=palettePico[c1*3+1];
        _palette[c0*3+2]=palettePico[c1*3+2];
        pb.display.loadRGBPalette(_palette);
        #elif defined PICOBOY
        _palette[c0]=palettePico[c1];
        pb.display.load565Palette(_palette);
        #endif // POKITTO
    }
    //else
    //unsupported
};
void palt (uint8_t c, bool t)
{
    if(t)
    {
        #ifdef POKITTO
        _palette[c*3]=palettePico[pb.display.getInvisibleColor()*3];
        _palette[c*3+1]=palettePico[pb.display.getInvisibleColor()*3+1];
        _palette[c*3+1]=palettePico[pb.display.getInvisibleColor()*3+2];
        pb.display.loadRGBPalette(_palette);
        #elif defined PICOBOY
        _palette[c]=palettePico[pb.display.getTransparentColor()];
        pb.display.load565Palette(_palette);
        #endif // POKITTO
    }else
    {
        #ifdef POKITTO
        _palette[c*3]=palettePico[c*3];
        _palette[c*3+1]=palettePico[c*3+1];
        _palette[c*3+1]=palettePico[c*3+2];
        pb.display.loadRGBPalette(_palette);
        #elif defined PICOBOY
        _palette[c]=palettePico[c];
        pb.display.load565Palette(_palette);
        #endif // POKITTO
    }
};

void pal ()
{
    #ifdef POKITTO
        pb.display.loadRGBPalette(palettePico);
    #elif defined PICOBOY
        pb.display.load565Palette(palettePico);
    #endif // POKITTO
};

void palt ()
{
    #ifdef POKITTO
        pb.display.setInvisibleColor(BLACK);
    #elif defined PICOBOY
        pb.display.setTransparentColor(BLACK);
    #endif // POKITTO
};

void spr(uint8_t n, uint8_t x, uint8_t y ,uint8_t w ,uint8_t h ,bool flip_x, bool flip_y)
{
    uint8_t flip=0;
    if(flip_x)
      flip |= 1;
    if(flip_y)
      flip |= 2;
   if(flip)
   {
    #ifdef POKITTO
       pb.display.drawBitmap(X_OFFSET+x,Y_OFFSET+y,_gfx[n],0,flip);
    #elif defined PICOBOY
       pb.display.drawBitmapRot(x,y,_gfx[n],0,flip);
    #endif
   }

   else
    pb.display.drawBitmap(x,y,_gfx[n]);
}


/*--------------------------------------------------------------------------------------------
	Input
--------------------------------------------------------------------------------------------*/
bool btn (uint8_t i, uint8_t p)
{
    #ifdef POKITTO
        switch(i)
        {
        case 0:
            return pb.buttons.leftBtn();
         case 1:
            return pb.buttons.rightBtn();
        case 2:
            return pb.buttons.upBtn();
        case 3:
            return pb.buttons.downBtn();
        case 4:
            return pb.buttons.aBtn();
        case 5:
            return pb.buttons.bBtn();
        case 6:
            return pb.buttons.cBtn();
        }
    #elif defined PICOBOY
        return pb.buttons.repeat(i,0);
    #endif // POKITTO

};
bool pbtn (uint8_t i, uint8_t p)
{
    #ifdef POKITTO
        switch(i)
        {
        case 0:
            return pb.buttons.leftBtn();
         case 1:
            return pb.buttons.rightBtn();
        case 2:
            return pb.buttons.upBtn();
        case 3:
            return pb.buttons.downBtn();
        case 4:
            return pb.buttons.aBtn();
        case 5:
            return pb.buttons.bBtn();
        case 6:
            return pb.buttons.cBtn();
        }
    #elif defined PICOBOY
        return pb.buttons.pressed(i);
    #endif // POKITTO
};


/*--------------------------------------------------------------------------------------------
	Audio
--------------------------------------------------------------------------------------------*/
void sfx (uint8_t n, uint8_t channel , int16_t offset , uint16_t length)
{
    //TODO
};
void music (uint8_t n, uint8_t fade_len ,uint8_t channel_mask)
{
    //TODO
};

/*--------------------------------------------------------------------------------------------------------
	Map
--------------------------------------------------------------------------------------------------------*/
uint8_t mget (uint8_t x,uint8_t y)
{
    return _map[y][x];
};
void mset (uint8_t x,uint8_t y, uint8_t v)
{
    //unsupported
};
void map(uint8_t cel_x, uint8_t cel_y, uint8_t x, uint8_t y, uint8_t cel_w, uint8_t cel_h, uint8_t layer)
{
    for(uint8_t i=0; i< cel_h; i++)
            for(uint8_t j=0; j<cel_w; j++)
            {
                uint8_t tile=_map[cel_y+i][cel_x+j];
                if(tile==0)
                  continue;

                if(layer==0)
                  pb.display.drawBitmap(X_OFFSET+x+(j*8),Y_OFFSET+y+(i*8),_gfx[tile]);
                else
                {
                  uint8_t flag = fget(tile);
                  for(uint8_t k=1; k<9; k++)
                  {
                    if((flag&layer)&(1<<(k-1)))
                      pb.display.drawBitmap(X_OFFSET+x+(j*8),Y_OFFSET+y+(i*8),_gfx[tile]);
                  }
                }
            }
};

/*--------------------------------------------------------------------------------------------------------
	Math
-------------------------------------------------------------------------------------------------------*/
int16_t rnd(int16_t x)
{
    return random(x);
};

